@@ -2,7 +2,7 @@
 PC-BASIC - bytematrix.py
 2D matrices of bytes
 
-(c) 2018--2021 Rob Hagemans
+(c) 2018--2022 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -204,7 +204,7 @@ class ByteMatrix(object):
     @classmethod
     def frompacked(cls, packed, height, items_per_byte):
         """Unpack from packed-bits representation."""
-        packed = bytes(packed)
+        packed = bytes(bytearray(packed))
         if not packed or not height:
             return cls(0, 0)
         width = len(packed) // height
