@@ -2,7 +2,7 @@
 PC-BASIC - display.display
 Display and video mode operations
 
-(c) 2013--2021 Rob Hagemans
+(c) 2013--2022 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -72,9 +72,7 @@ class Display(object):
         # text screen
         self._codepage = codepage
         self.cursor = Cursor(queues, self.mode)
-        self.text_screen = TextScreen(
-            self._queues, self._values, self.mode, self.cursor, self._adapter
-        )
+        self.text_screen = TextScreen(self._values, self.mode, self.cursor, self._adapter)
         # page buffers, set by _set_mode
         self.pages = None
         # screen aspect ratio: used to determine pixel aspect ratio, which is used by CIRCLE
@@ -281,7 +279,6 @@ class Display(object):
         # note that we *must* rebuild the list of pages if we have more pages
         # and also we must do something if we have fewer pages and vpage or apage are currently on a higher page.
         # apparently pcjr always drops to text when this is not a no-op
-        print('here')
         self.screen(0, 0, 0, 0, force_reset=True)
 
     def rebuild(self):
