@@ -2,7 +2,7 @@
 PC-BASIC - display.framebuffer
 Emulated video memory
 
-(c) 2013--2021 Rob Hagemans
+(c) 2013--2022 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -180,14 +180,18 @@ class Tandy6SpriteBuilder(PlanedSpriteBuilder):
     def pack(self, sprite):
         """Pack sprite, report only half the width packed."""
         record = PlanedSpriteBuilder.pack(self, sprite)
-        width_record = struct.pack('<H', sprite.width // 2)
+        width = sprite.width // self.width_factor
+        width_record = struct.pack('<H', width)
         return width_record + record[2:]
 
     def unpack(self, array):
         """Unpack sprite, twice the width reported."""
-        width = 2 * struct.unpack('<H', array[:2])
+        width, = struct.unpack('<H', array[:2])
+        width *= self.width_factor
         size_record = struct.pack('<H', width)
-        return PlanedSpriteBuilder.unpack(self, size_record + array[2:])
+        # explicit conversion to bytes only needed for Python 2, in Python 3 concatenation is ok
+        # via bytearray as otherwide it gives str representation in Python 2
+        return PlanedSpriteBuilder.unpack(self, size_record + bytes(bytearray(array[2:])))
 
 
 ##############################################################################
