@@ -2,7 +2,7 @@
 PC-BASIC - config.py
 Configuration file and command-line options parser
 
-(c) 2013--2021 Rob Hagemans
+(c) 2013--2022 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -238,9 +238,7 @@ def _check_text_encoding(arg):
 
 def _check_max_memory(arglist):
     """Check if max-memory argument is acceptable."""
-
     mem_sizes = [arglist[0], arglist[1]*16 if arglist[1] else None]
-
     if min((mem_size for mem_size in mem_sizes if mem_size), default=MAX_MEMORY_SIZE) > MAX_MEMORY_SIZE:
         logging.warning(u'max-memory value > %s', MAX_MEMORY_SIZE)
         return False
@@ -261,8 +259,8 @@ ARGUMENTS = {
         ),
     },
     u'sound': {
-        u'type': u'string', u'default': u'',
-        u'choices': (u'', u'none', u'beep', u'portaudio', u'sdl2', u'interface'),
+        u'type': u'string', u'default': u'true',
+        u'choices': (u'true', u'false', u'none', u'beep', u'portaudio', u'sdl2', u'interface'),
     },
     u'load': {u'type': u'string', u'default': u'', },
     u'run': {u'type': u'string', u'default': u'',  },
@@ -494,7 +492,7 @@ class Settings(object):
         """Get value of option; choose whether to get default or None (unspecified) or '' (empty)."""
         try:
             value = self._options[name]
-            if get_default and (value is None or value == u''):
+            if get_default and (value is None or value == u'' or value == []):
                 raise KeyError
         except KeyError:
             if get_default:
@@ -572,7 +570,7 @@ class Settings(object):
             'hide_listing': self.get('hide-listing'),
             'hide_protected': self.get('hide-protected'),
             'allow_code_poke': self.get('allow-code-poke'),
-            'rebuild_offsets': not self.get('convert'),
+            'rebuild_offsets': not self.convert,
             # max available memory to BASIC (set by /m)
             'max_memory': min(max_list) or 65534,
             # maximum record length (-s)
@@ -673,6 +671,9 @@ class Settings(object):
             mount_dict = self._get_drives_from_list(mount_list)
         # directory for bundled BASIC programs accessible through @:
         mount_dict[b'@'] = PROGRAM_PATH
+        # if Z: not specified, override it to avoid mounting through Session default
+        if b'Z' not in mount_dict:
+            mount_dict[b'Z'] = None
         return current_device, mount_dict
 
     def _get_drives_from_list(self, mount_list):
@@ -769,7 +770,7 @@ class Settings(object):
                 iface_list = (interface,)
         iface_params = {
             'try_interfaces': iface_list,
-            'audio_override': self.get('sound') != 'interface' and self.get('sound'),
+            'audio_override': self.get('sound') not in ('true', 'interface') and self.get('sound'),
         }
         iface_params.update(self._get_video_parameters())
         iface_params.update(self._get_audio_parameters())
@@ -858,8 +859,8 @@ class Settings(object):
     def conv_params(self):
         """Get parameters for file conversion."""
         # conversion output
-        # first arg, if given, is mode; second arg, if given, is outfile
-        mode = self.get('convert')
+        # argument is mode
+        mode = self.get('convert', get_default=False)
         # keep uppercase first letter
         mode = mode[0].upper() if mode else 'A'
         name_in = (self.get(0) or self.get('run') or self.get('load'))
@@ -879,7 +880,7 @@ class Settings(object):
     @property
     def convert(self):
         """Converter operating mode."""
-        return self.get('convert')
+        return self.get('convert', get_default=False) is not None
 
     @property
     def debug(self):
