@@ -2,7 +2,7 @@
 PC-BASIC test.session
 unit tests for session API
 
-(c) 2020--2021 Rob Hagemans
+(c) 2020--2022 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -40,11 +40,12 @@ class SessionTest(TestCase):
             # unset array
             s.set_variable('A%()', [[0,0,5], [0,0,6]])
             assert s.get_variable('A%()') == [
-                [0, 0, 5, 0, 0, 0, 0, 0, 0, 0], [0, 0, 6, 0, 0, 0, 0, 0, 0, 0],
-                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
+                [0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 6, 0, 0, 0, 0, 0, 0, 0, 0],
+                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
             ]
             assert s.evaluate('A%(0,2)') == 5
             assert s.evaluate('A%(1,2)') == 6
@@ -295,6 +296,34 @@ class SessionTest(TestCase):
         with open(self.output_path('print.txt')) as f:
             assert f.read() == ''
 
+    def test_gosub_from_direct_line(self):
+        """Test for issue#184: GOSUB from direct line should not RETURN into program."""
+        SOURCE = """\
+        10 PRINT "Main"
+        30 A = -42
+        40 END
+        50 PRINT "After End"
+        60 A = 42
+        70 RETURN
+        """
+        with Session() as session:
+            session.execute(SOURCE)
+            session.execute("GOSUB 60")
+            assert session.evaluate('A') == 42
+
+    def test_to_list_off_by_one(self):
+        """Test for issue #182: range off by one in to_list."""
+        with Session() as session:
+            session.execute("""
+            DIM J2(5,2)
+            J2(1,1)=-1
+            J2(1,2)=-1
+            """)
+            assert session.get_variable('J2!()') == [
+                [0.0, 0.0, 0.0], [0.0, -1.0, -1.0], [0.0, 0.0, 0.0],
+                [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]
+            ]
+
 
 from pcbasic.basic import iostreams
 from pcbasic.basic.codepage import Codepage
